@@ -219,6 +219,15 @@ class Value(GObjectModule.Value):
         if self._free_on_dealloc and self.g_type != TYPE_INVALID:
             self.unset()
 
+    def set_boxed(self, boxed):
+        # Workaround the introspection marshalers inability to know
+        # these methods should be marshaling boxed types. This is because
+        # the type information is stored on the GValue.
+        _gobject._gvalue_set(self, boxed)
+
+    def get_boxed(self):
+        return _gobject._gvalue_get(self)
+
     def set_value(self, py_value):
         gtype = self.g_type
 
@@ -448,8 +457,8 @@ def signal_handler_block(obj, handler_id):
     Returns a context manager which optionally can be used to
     automatically unblock the handler:
 
-    >>> with GObject.signal_handler_block(obj, id):
-    >>>    pass
+      with GObject.signal_handler_block(obj, id):
+         pass
     """
     GObjectModule.signal_handler_block(_get_instance_for_signal(obj), handler_id)
     return _HandlerBlockManager(obj, handler_id)
@@ -616,8 +625,8 @@ class Object(GObjectModule.Object):
         Returns a context manager which optionally can be used to
         automatically thaw notifications:
 
-        >>> with obj.freeze_notify():
-        >>>     pass
+          with obj.freeze_notify():
+              pass
         """
         super(Object, self).freeze_notify()
         return _FreezeNotifyManager(self)
