@@ -33,6 +33,9 @@ _pygi_closure_assign_pyobj_to_retval (gpointer retval, PyObject *object,
                                       GITransfer transfer)
 {
     GIArgument arg = _pygi_argument_from_object (object, type_info, transfer);
+    if (PyErr_Occurred ())
+        return;
+
     GITypeTag type_tag = g_type_info_get_tag (type_info);
 
     if (retval == NULL)
@@ -99,6 +102,7 @@ _pygi_closure_assign_pyobj_to_retval (gpointer retval, PyObject *object,
                 }
 
                 g_base_info_unref (interface_info);
+                break;
             }
         default:
             *(ffi_arg *) retval = (ffi_arg) arg.v_pointer;
@@ -106,6 +110,19 @@ _pygi_closure_assign_pyobj_to_retval (gpointer retval, PyObject *object,
       }
 }
 
+static void
+_pygi_closure_clear_retval (GICallableInfo *callable_info, gpointer retval)
+{
+    GITypeInfo return_type_info;
+    GITypeTag return_type_tag;
+
+    g_callable_info_load_return_type (callable_info, &return_type_info);
+    return_type_tag = g_type_info_get_tag (&return_type_info);
+    if (return_type_tag != GI_TYPE_TAG_VOID) {
+        *((ffi_arg *) retval) = 0;
+    }
+}
+
 static void
 _pygi_closure_assign_pyobj_to_out_argument (gpointer out_arg, PyObject *object,
                                             GITypeInfo *type_info,
@@ -271,7 +288,7 @@ _pygi_closure_convert_ffi_arguments (GICallableInfo *callable_info, void **args)
                         break;
                     } else if (interface_type == GI_INFO_TYPE_ENUM ||
                                interface_type == GI_INFO_TYPE_FLAGS) {
-                        g_args[i].v_double = * (double *) args[i];
+                        g_args[i].v_uint = * (guint *) args[i];
                         g_base_info_unref (interface);
                         break;
                     } else if (interface_type == GI_INFO_TYPE_STRUCT ||
@@ -504,15 +521,8 @@ _pygi_invoke_closure_clear_py_data(PyGICClosure *invoke_closure)
 {
     PyGILState_STATE state = PyGILState_Ensure();
 
-    if (invoke_closure->function != NULL) {
-        Py_DECREF (invoke_closure->function);
-        invoke_closure->function = NULL;
-    }
-
-    if (invoke_closure->user_data != NULL) {
-        Py_DECREF (invoke_closure->user_data);
-        invoke_closure->user_data = NULL;
-    }
+    Py_CLEAR (invoke_closure->function);
+    Py_CLEAR (invoke_closure->user_data);
 
     PyGILState_Release (state);
 }
@@ -545,11 +555,17 @@ _pygi_closure_handle (ffi_cif *cif,
     Py_DECREF (py_args);
 
     if (retval == NULL) {
+        _pygi_closure_clear_retval (closure->info, result);
         PyErr_Print();
         goto end;
     }
 
     _pygi_closure_set_out_arguments (closure->info, retval, out_args, result);
+    if (PyErr_Occurred ()) {
+        _pygi_closure_clear_retval (closure->info, result);
+        PyErr_Print();
+    }
+
     Py_DECREF (retval);
 
 end:
