@@ -251,434 +251,30 @@ _pygi_marshal_from_py_void (PyGIInvokeState   *state,
 {
     g_warn_if_fail (arg_cache->transfer == GI_TRANSFER_NOTHING);
 
-    if (PYGLIB_CPointer_Check(py_arg)) {
+    if (py_arg == Py_None) {
+        arg->v_pointer = NULL;
+    } else if (PYGLIB_CPointer_Check(py_arg)) {
         arg->v_pointer = PYGLIB_CPointer_GetPointer (py_arg, NULL);
     } else {
+        /* NOTE: This will change to only allow integers and the deprecation
+         * warning will become a runtime exception. Using the following:
+         * arg->v_pointer = PyLong_AsVoidPtr (py_arg);
+         * See: https://bugzilla.gnome.org/show_bug.cgi?id=688081
+         */
+
+        if (!PYGLIB_PyLong_Check(py_arg) && !PyLong_Check(py_arg)) {
+            if (PyErr_WarnEx(PyGIDeprecationWarning,
+                             "Pointer arguments will be restricted to integers, capsules, and None. "
+                             "See: https://bugzilla.gnome.org/show_bug.cgi?id=683599",
+                             1))
+                return FALSE;
+        }
         arg->v_pointer = py_arg;
     }
 
     return TRUE;
 }
 
-gboolean
-_pygi_marshal_from_py_boolean (PyGIInvokeState   *state,
-                               PyGICallableCache *callable_cache,
-                               PyGIArgCache      *arg_cache,
-                               PyObject          *py_arg,
-                               GIArgument        *arg)
-{
-    arg->v_boolean = PyObject_IsTrue (py_arg);
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_int8 (PyGIInvokeState   *state,
-                            PyGICallableCache *callable_cache,
-                            PyGIArgCache      *arg_cache,
-                            PyObject          *py_arg,
-                            GIArgument        *arg)
-{
-    PyObject *py_long;
-    long long_;
-
-    if (PYGLIB_PyBytes_Check (py_arg)) {
-
-        if (PYGLIB_PyBytes_Size (py_arg) != 1) {
-            PyErr_Format (PyExc_TypeError, "Must be a single character");
-            return FALSE;
-        }
-
-        long_ = (char)(PYGLIB_PyBytes_AsString (py_arg)[0]);
-    } else if (PyNumber_Check (py_arg)) {
-        py_long = PYGLIB_PyNumber_Long (py_arg);
-        if (!py_long)
-            return FALSE;
-
-        long_ = PYGLIB_PyLong_AsLong (py_long);
-        Py_DECREF (py_long);
-
-        if (PyErr_Occurred ()) {
-            PyErr_Clear ();
-            PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, -128, 127);
-            return FALSE;
-        }
-    } else {
-        PyErr_Format (PyExc_TypeError, "Must be number or single byte string, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    if (long_ < -128 || long_ > 127) {
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, -128, 127);
-        return FALSE;
-    }
-
-    arg->v_int8 = long_;
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_uint8 (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             PyObject          *py_arg,
-                             GIArgument        *arg)
-{
-    unsigned long long_;
-
-    if (PYGLIB_PyBytes_Check (py_arg)) {
-
-        if (PYGLIB_PyBytes_Size (py_arg) != 1) {
-            PyErr_Format (PyExc_TypeError, "Must be a single character");
-            return FALSE;
-        }
-
-        long_ = (unsigned char)(PYGLIB_PyBytes_AsString (py_arg)[0]);
-
-    } else if (PyNumber_Check (py_arg)) {
-        PyObject *py_long;
-        py_long = PYGLIB_PyNumber_Long (py_arg);
-        if (!py_long)
-            return FALSE;
-
-        long_ = PYGLIB_PyLong_AsLong (py_long);
-        Py_DECREF (py_long);
-
-        if (PyErr_Occurred ()) {
-            PyErr_Clear();
-
-            PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, 0, 255);
-            return FALSE;
-        }
-    } else {
-        PyErr_Format (PyExc_TypeError, "Must be number or single byte string, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    if (long_ < 0 || long_ > 255) {
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, 0, 255);
-        return FALSE;
-    }
-
-    arg->v_uint8 = long_;
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_int16 (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             PyObject          *py_arg,
-                             GIArgument        *arg)
-{
-    PyObject *py_long;
-    long long_;
-
-    if (!PyNumber_Check (py_arg)) {
-        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    py_long = PYGLIB_PyNumber_Long (py_arg);
-    if (!py_long)
-        return FALSE;
-
-    long_ = PYGLIB_PyLong_AsLong (py_long);
-    Py_DECREF (py_long);
-
-    if (PyErr_Occurred ()) {
-        PyErr_Clear ();
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, -32768, 32767);
-        return FALSE;
-    }
-
-    if (long_ < -32768 || long_ > 32767) {
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, -32768, 32767);
-        return FALSE;
-    }
-
-    arg->v_int16 = long_;
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_uint16 (PyGIInvokeState   *state,
-                              PyGICallableCache *callable_cache,
-                              PyGIArgCache      *arg_cache,
-                              PyObject          *py_arg,
-                              GIArgument        *arg)
-{
-    PyObject *py_long;
-    long long_;
-
-    if (!PyNumber_Check (py_arg)) {
-        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    py_long = PYGLIB_PyNumber_Long (py_arg);
-    if (!py_long)
-        return FALSE;
-
-    long_ = PYGLIB_PyLong_AsLong (py_long);
-    Py_DECREF (py_long);
-
-    if (PyErr_Occurred ()) {
-        PyErr_Clear ();
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, 0, 65535);
-        return FALSE;
-    }
-
-    if (long_ < 0 || long_ > 65535) {
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, 0, 65535);
-        return FALSE;
-    }
-
-    arg->v_uint16 = long_;
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_int32 (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             PyObject          *py_arg,
-                             GIArgument        *arg)
-{
-    PyObject *py_long;
-    long long_;
-
-    if (!PyNumber_Check (py_arg)) {
-        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    py_long = PYGLIB_PyNumber_Long (py_arg);
-    if (!py_long)
-        return FALSE;
-
-    long_ = PYGLIB_PyLong_AsLong (py_long);
-    Py_DECREF (py_long);
-
-    if (PyErr_Occurred ()) {
-        PyErr_Clear();
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, G_MININT32, G_MAXINT32);
-        return FALSE;
-    }
-
-    if (long_ < G_MININT32 || long_ > G_MAXINT32) {
-        PyErr_Format (PyExc_ValueError, "%ld not in range %d to %d", long_, G_MININT32, G_MAXINT32);
-        return FALSE;
-    }
-
-    arg->v_int32 = long_;
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_uint32 (PyGIInvokeState   *state,
-                              PyGICallableCache *callable_cache,
-                              PyGIArgCache      *arg_cache,
-                              PyObject          *py_arg,
-                              GIArgument        *arg)
-{
-    PyObject *py_long;
-    long long long_;
-
-    if (!PyNumber_Check (py_arg)) {
-        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    py_long = PYGLIB_PyNumber_Long (py_arg);
-    if (!py_long)
-        return FALSE;
-
-#if PY_VERSION_HEX < 0x03000000
-    if (PyInt_Check (py_long))
-        long_ = PyInt_AsLong (py_long);
-    else
-#endif
-        long_ = PyLong_AsLongLong (py_long);
-
-    Py_DECREF (py_long);
-
-    if (PyErr_Occurred ()) {
-        PyErr_Clear ();
-        PyErr_Format (PyExc_ValueError, "%lld not in range %i to %u", long_, 0, G_MAXUINT32);
-        return FALSE;
-    }
-
-    if (long_ < 0 || long_ > G_MAXUINT32) {
-        PyErr_Format (PyExc_ValueError, "%lld not in range %i to %u", long_, 0, G_MAXUINT32);
-        return FALSE;
-    }
-
-    arg->v_uint32 = long_;
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_int64 (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             PyObject          *py_arg,
-                             GIArgument        *arg)
-{
-    PyObject *py_long;
-    gint64 long_;
-
-    if (!PyNumber_Check (py_arg)) {
-        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    py_long = PYGLIB_PyNumber_Long (py_arg);
-    if (!py_long)
-        return FALSE;
-
-#if PY_VERSION_HEX < 0x03000000
-    if (PyInt_Check (py_long))
-        long_ = (gint64) PyInt_AS_LONG (py_long);
-    else
-#endif
-        long_ = (gint64) PyLong_AsLongLong (py_long);
-
-    Py_DECREF (py_long);
-
-    if (PyErr_Occurred ()) {
-        /* OverflowError occured but range errors should be returned as ValueError */
-        char *long_str;
-        PyObject *py_str;
-
-        PyErr_Clear ();
-
-        py_str = PyObject_Str (py_long);
-
-        if (PyUnicode_Check (py_str)) {
-            PyObject *py_bytes = PyUnicode_AsUTF8String (py_str);
-            Py_DECREF (py_str);
-
-            if (py_bytes == NULL)
-                return FALSE;
-
-            long_str = g_strdup (PYGLIB_PyBytes_AsString (py_bytes));
-            if (long_str == NULL) {
-                PyErr_NoMemory ();
-                return FALSE;
-            }
-
-            Py_DECREF (py_bytes);
-        } else {
-            long_str = g_strdup (PYGLIB_PyBytes_AsString(py_str));
-            Py_DECREF (py_str);
-        }
-
-        PyErr_Format (PyExc_ValueError, "%s not in range %lld to %lld",
-                      long_str, (long long) G_MININT64, (long long) G_MAXINT64);
-
-        g_free (long_str);
-        return FALSE;
-    }
-
-    arg->v_int64 = long_;
-
-    return TRUE;
-}
-
-gboolean
-_pygi_marshal_from_py_uint64 (PyGIInvokeState   *state,
-                              PyGICallableCache *callable_cache,
-                              PyGIArgCache      *arg_cache,
-                              PyObject          *py_arg,
-                              GIArgument        *arg)
-{
-    PyObject *py_long;
-    guint64 ulong_;
-
-    if (!PyNumber_Check (py_arg)) {
-        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
-                      py_arg->ob_type->tp_name);
-        return FALSE;
-    }
-
-    py_long = PYGLIB_PyNumber_Long (py_arg);
-    if (!py_long)
-        return FALSE;
-
-#if PY_VERSION_HEX < 0x03000000
-    if (PyInt_Check (py_long)) {
-        long long_ =  PyInt_AsLong (py_long);
-        if (long_ < 0 || long_ > G_MAXUINT64) {
-            PyErr_Format (PyExc_ValueError, "%" G_GUINT64_FORMAT " not in range %d to %" G_GUINT64_FORMAT,
-                          (gint64) long_, 0, G_MAXUINT64);
-            return FALSE;
-        }
-        ulong_ = (guint64) long_;
-    } else
-#endif
-        ulong_ = PyLong_AsUnsignedLongLong (py_long);
-
-    Py_DECREF (py_long);
-
-    if (PyErr_Occurred ()) {
-        /* OverflowError occured but range errors should be returned as ValueError */
-        char *long_str;
-        PyObject *py_str;
-
-        PyErr_Clear ();
-
-        py_str = PyObject_Str (py_long);
-
-        if (PyUnicode_Check (py_str)) {
-            PyObject *py_bytes = PyUnicode_AsUTF8String (py_str);
-            Py_DECREF (py_str);
-
-            if (py_bytes == NULL)
-                return FALSE;
-
-            long_str = g_strdup (PYGLIB_PyBytes_AsString (py_bytes));
-            if (long_str == NULL) {
-                PyErr_NoMemory ();
-                return FALSE;
-            }
-
-            Py_DECREF (py_bytes);
-        } else {
-            long_str = g_strdup (PYGLIB_PyBytes_AsString (py_str));
-            Py_DECREF (py_str);
-        }
-
-        PyErr_Format (PyExc_ValueError, "%s not in range %d to %" G_GUINT64_FORMAT,
-                      long_str, 0, G_MAXUINT64);
-
-        g_free (long_str);
-        return FALSE;
-    }
-
-    if (ulong_ > G_MAXUINT64) {
-        PyErr_Format (PyExc_ValueError, "%" G_GUINT64_FORMAT " not in range %d to %" G_GUINT64_FORMAT, ulong_, 0, G_MAXUINT64);
-        return FALSE;
-    }
-
-    arg->v_uint64 = ulong_;
-
-    return TRUE;
-}
-
 static gboolean
 check_valid_double (double x, double min, double max)
 {
@@ -690,7 +286,7 @@ check_valid_double (double x, double min, double max)
 
         /* we need this as PyErr_Format() does not support float types */
         snprintf (buf, sizeof (buf), "%g not in range %g to %g", x, min, max);
-        PyErr_SetString (PyExc_ValueError, buf);
+        PyErr_SetString (PyExc_OverflowError, buf);
         return FALSE;
     }
     return TRUE;
@@ -718,11 +314,8 @@ _pygi_py_arg_to_double (PyObject *py_arg, double *double_)
     return TRUE;
 }
 
-gboolean
-_pygi_marshal_from_py_float (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             PyObject          *py_arg,
+static gboolean
+_pygi_marshal_from_py_float (PyObject          *py_arg,
                              GIArgument        *arg)
 {
     double double_;
@@ -737,11 +330,8 @@ _pygi_marshal_from_py_float (PyGIInvokeState   *state,
     return TRUE;
 }
 
-gboolean
-_pygi_marshal_from_py_double (PyGIInvokeState   *state,
-                              PyGICallableCache *callable_cache,
-                              PyGIArgCache      *arg_cache,
-                              PyObject          *py_arg,
+static gboolean
+_pygi_marshal_from_py_double (PyObject          *py_arg,
                               GIArgument        *arg)
 {
     double double_;
@@ -756,11 +346,8 @@ _pygi_marshal_from_py_double (PyGIInvokeState   *state,
     return TRUE;
 }
 
-gboolean
-_pygi_marshal_from_py_unichar (PyGIInvokeState   *state,
-                               PyGICallableCache *callable_cache,
-                               PyGIArgCache      *arg_cache,
-                               PyObject          *py_arg,
+static gboolean
+_pygi_marshal_from_py_unichar (PyObject          *py_arg,
                                GIArgument        *arg)
 {
     Py_ssize_t size;
@@ -811,11 +398,8 @@ _pygi_marshal_from_py_unichar (PyGIInvokeState   *state,
     return TRUE;
 }
 
-gboolean
-_pygi_marshal_from_py_gtype (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             PyObject          *py_arg,
+static gboolean
+_pygi_marshal_from_py_gtype (PyObject          *py_arg,
                              GIArgument        *arg)
 {
     long type_ = pyg_type_from_object (py_arg);
@@ -829,11 +413,9 @@ _pygi_marshal_from_py_gtype (PyGIInvokeState   *state,
     arg->v_long = type_;
     return TRUE;
 }
-gboolean
-_pygi_marshal_from_py_utf8 (PyGIInvokeState   *state,
-                            PyGICallableCache *callable_cache,
-                            PyGIArgCache      *arg_cache,
-                            PyObject          *py_arg,
+
+static gboolean
+_pygi_marshal_from_py_utf8 (PyObject          *py_arg,
                             GIArgument        *arg)
 {
     gchar *string_;
@@ -866,11 +448,8 @@ _pygi_marshal_from_py_utf8 (PyGIInvokeState   *state,
     return TRUE;
 }
 
-gboolean
-_pygi_marshal_from_py_filename (PyGIInvokeState   *state,
-                                PyGICallableCache *callable_cache,
-                                PyGIArgCache      *arg_cache,
-                                PyObject          *py_arg,
+static gboolean
+_pygi_marshal_from_py_filename (PyObject          *py_arg,
                                 GIArgument        *arg)
 {
     gchar *string_;
@@ -908,6 +487,237 @@ _pygi_marshal_from_py_filename (PyGIInvokeState   *state,
     return TRUE;
 }
 
+static gboolean
+_pygi_marshal_from_py_long (PyObject   *object,   /* in */
+                            GIArgument *arg,      /* out */
+                            GITypeTag   type_tag,
+                            GITransfer  transfer)
+{
+    PyObject *number;
+
+    if (!PyNumber_Check (object)) {
+        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
+                      object->ob_type->tp_name);
+        return FALSE;
+    }
+
+#if PY_MAJOR_VERSION < 3
+    {
+        PyObject *tmp = PyNumber_Int (object);
+        if (tmp) {
+            number = PyNumber_Long (tmp);
+            Py_DECREF (tmp);
+        } else {
+            number = PyNumber_Long (object);
+        }
+    }
+#else
+    number = PyNumber_Long (object);
+#endif
+
+    if (number == NULL) {
+        PyErr_SetString (PyExc_TypeError, "expected int argument");
+        return FALSE;
+    }
+
+    switch (type_tag) {
+        case GI_TYPE_TAG_INT8:
+        {
+            long long_value = PyLong_AsLong (number);
+            if (PyErr_Occurred()) {
+                break;
+            } else if (long_value < G_MININT8 || long_value > G_MAXINT8) {
+                PyErr_Format (PyExc_OverflowError, "%ld not in range %ld to %ld",
+                              long_value, (long)G_MININT8, (long)G_MAXINT8);
+            } else {
+                arg->v_int8 = long_value;
+            }
+            break;
+        }
+
+        case GI_TYPE_TAG_UINT8:
+        {
+            long long_value = PyLong_AsLong (number);
+            if (PyErr_Occurred()) {
+                break;
+            } else if (long_value < 0 || long_value > G_MAXUINT8) {
+                PyErr_Format (PyExc_OverflowError, "%ld not in range %ld to %ld",
+                              long_value, (long)0, (long)G_MAXUINT8);
+            } else {
+                arg->v_uint8 = long_value;
+            }
+            break;
+        }
+
+        case GI_TYPE_TAG_INT16:
+        {
+            long long_value = PyLong_AsLong (number);
+            if (PyErr_Occurred()) {
+                break;
+            } else if (long_value < G_MININT16 || long_value > G_MAXINT16) {
+                PyErr_Format (PyExc_OverflowError, "%ld not in range %ld to %ld",
+                              long_value, (long)G_MININT16, (long)G_MAXINT16);
+            } else {
+                arg->v_int16 = long_value;
+            }
+            break;
+        }
+
+        case GI_TYPE_TAG_UINT16:
+        {
+            long long_value = PyLong_AsLong (number);
+            if (PyErr_Occurred()) {
+                break;
+            } else if (long_value < 0 || long_value > G_MAXUINT16) {
+                PyErr_Format (PyExc_OverflowError, "%ld not in range %ld to %ld",
+                              long_value, (long)0, (long)G_MAXUINT16);
+            } else {
+                arg->v_uint16 = long_value;
+            }
+            break;
+        }
+
+        case GI_TYPE_TAG_INT32:
+        {
+            long long_value = PyLong_AsLong (number);
+            if (PyErr_Occurred()) {
+                break;
+            } else if (long_value < G_MININT32 || long_value > G_MAXINT32) {
+                PyErr_Format (PyExc_OverflowError, "%ld not in range %ld to %ld",
+                              long_value, (long)G_MININT32, (long)G_MAXINT32);
+            } else {
+                arg->v_int32 = long_value;
+            }
+            break;
+        }
+
+        case GI_TYPE_TAG_UINT32:
+        {
+            PY_LONG_LONG long_value = PyLong_AsLongLong (number);
+            if (PyErr_Occurred()) {
+                break;
+            } else if (long_value < 0 || long_value > G_MAXUINT32) {
+                PyErr_Format (PyExc_OverflowError, "%lld not in range %ld to %lu",
+                              long_value, (long)0, (unsigned long)G_MAXUINT32);
+            } else {
+                arg->v_uint32 = long_value;
+            }
+            break;
+        }
+
+        case GI_TYPE_TAG_INT64:
+        {
+            /* Rely on Python overflow error and convert to ValueError for 64 bit values */
+            arg->v_int64 = PyLong_AsLongLong (number);
+            break;
+        }
+
+        case GI_TYPE_TAG_UINT64:
+        {
+            /* Rely on Python overflow error and convert to ValueError for 64 bit values */
+            arg->v_uint64 = PyLong_AsUnsignedLongLong (number);
+            break;
+        }
+
+        default:
+            g_assert_not_reached ();
+    }
+
+    Py_DECREF (number);
+
+    if (PyErr_Occurred())
+        return FALSE;
+    return TRUE;
+}
+
+gboolean
+_pygi_marshal_from_py_basic_type (PyObject   *object,   /* in */
+                                  GIArgument *arg,      /* out */
+                                  GITypeTag   type_tag,
+                                  GITransfer  transfer)
+{
+    switch (type_tag) {
+        case GI_TYPE_TAG_VOID:
+            g_warn_if_fail (transfer == GI_TRANSFER_NOTHING);
+            if (object == Py_None) {
+                arg->v_pointer = NULL;
+            } else if (!PYGLIB_PyLong_Check(object)  && !PyLong_Check(object)) {
+                PyErr_SetString(PyExc_TypeError,
+                    "Pointer assignment is restricted to integer values. "
+                    "See: https://bugzilla.gnome.org/show_bug.cgi?id=683599");
+            } else {
+                arg->v_pointer = PyLong_AsVoidPtr (object);
+            }
+            break;
+        case GI_TYPE_TAG_INT8:
+        case GI_TYPE_TAG_UINT8:
+            if (PYGLIB_PyBytes_Check (object)) {
+                if (PYGLIB_PyBytes_Size (object) != 1) {
+                    PyErr_Format (PyExc_TypeError, "Must be a single character");
+                    return FALSE;
+                }
+                if (type_tag == GI_TYPE_TAG_INT8) {
+                    arg->v_int8 = (gint8)(PYGLIB_PyBytes_AsString (object)[0]);
+                } else {
+                    arg->v_uint8 = (guint8)(PYGLIB_PyBytes_AsString (object)[0]);
+                }
+            } else {
+                return _pygi_marshal_from_py_long (object, arg, type_tag, transfer);
+            }
+            break;
+        case GI_TYPE_TAG_INT16:
+        case GI_TYPE_TAG_UINT16:
+        case GI_TYPE_TAG_INT32:
+        case GI_TYPE_TAG_UINT32:
+        case GI_TYPE_TAG_INT64:
+        case GI_TYPE_TAG_UINT64:
+            return _pygi_marshal_from_py_long (object, arg, type_tag, transfer);
+
+        case GI_TYPE_TAG_BOOLEAN:
+            arg->v_boolean = PyObject_IsTrue (object);
+            break;
+
+        case GI_TYPE_TAG_FLOAT:
+            return _pygi_marshal_from_py_float (object, arg);
+
+        case GI_TYPE_TAG_DOUBLE:
+            return _pygi_marshal_from_py_double (object, arg);
+
+        case GI_TYPE_TAG_GTYPE:
+            return _pygi_marshal_from_py_gtype (object, arg);
+
+        case GI_TYPE_TAG_UNICHAR:
+            return _pygi_marshal_from_py_unichar (object, arg);
+
+        case GI_TYPE_TAG_UTF8:
+            return _pygi_marshal_from_py_utf8 (object, arg);
+
+        case GI_TYPE_TAG_FILENAME:
+            return _pygi_marshal_from_py_filename (object, arg);
+
+        default:
+            return FALSE;
+    }
+
+    if (PyErr_Occurred())
+        return FALSE;
+
+    return TRUE;
+}
+
+gboolean
+_pygi_marshal_from_py_basic_type_cache_adapter (PyGIInvokeState   *state,
+                                                PyGICallableCache *callable_cache,
+                                                PyGIArgCache      *arg_cache,
+                                                PyObject          *py_arg,
+                                                GIArgument        *arg)
+{
+    return _pygi_marshal_from_py_basic_type (py_arg,
+                                             arg,
+                                             arg_cache->type_tag,
+                                             arg_cache->transfer);
+}
+
 gboolean
 _pygi_marshal_from_py_array (PyGIInvokeState   *state,
                              PyGICallableCache *callable_cache,
@@ -1536,7 +1346,7 @@ _pygi_marshal_from_py_interface_enum (PyGIInvokeState   *state,
     long c_long;
     gint is_instance;
     PyGIInterfaceCache *iface_cache = (PyGIInterfaceCache *)arg_cache;
-    GIBaseInfo *interface;
+    GIBaseInfo *interface = NULL;
 
     is_instance = PyObject_IsInstance (py_arg, iface_cache->py_type);
 
@@ -1556,6 +1366,7 @@ _pygi_marshal_from_py_interface_enum (PyGIInvokeState   *state,
                                  c_long,
                                  g_enum_info_get_storage_type ((GIEnumInfo *)interface))) {
           g_assert_not_reached();
+          g_base_info_unref (interface);
           return FALSE;
     }
 
@@ -1581,9 +1392,12 @@ _pygi_marshal_from_py_interface_enum (PyGIInvokeState   *state,
             goto err;
     }
 
+    g_base_info_unref (interface);
     return TRUE;
 
 err:
+    if (interface)
+        g_base_info_unref (interface);
     PyErr_Format (PyExc_TypeError, "Expected a %s, but got %s",
                   iface_cache->type_name, py_arg->ob_type->tp_name);
     return FALSE;
@@ -1622,9 +1436,11 @@ _pygi_marshal_from_py_interface_flags (PyGIInvokeState   *state,
     g_assert (g_base_info_get_type (interface) == GI_INFO_TYPE_FLAGS);
     if (!gi_argument_from_c_long(arg, c_long,
                                  g_enum_info_get_storage_type ((GIEnumInfo *)interface))) {
+        g_base_info_unref (interface);
         return FALSE;
     }
 
+    g_base_info_unref (interface);
     return TRUE;
 
 err:
@@ -1635,24 +1451,24 @@ err:
 }
 
 gboolean
-_pygi_marshal_from_py_interface_struct (PyGIInvokeState   *state,
-                                        PyGICallableCache *callable_cache,
-                                        PyGIArgCache      *arg_cache,
-                                        PyObject          *py_arg,
-                                        GIArgument        *arg)
+_pygi_marshal_from_py_interface_struct_cache_adapter (PyGIInvokeState   *state,
+                                                      PyGICallableCache *callable_cache,
+                                                      PyGIArgCache      *arg_cache,
+                                                      PyObject          *py_arg,
+                                                      GIArgument        *arg)
 {
     PyGIInterfaceCache *iface_cache = (PyGIInterfaceCache *)arg_cache;
 
-    return pygi_marshal_from_py_interface_struct (py_arg,
-                                                  arg,
-                                                  arg_cache->arg_name,
-                                                  iface_cache->interface_info,
-                                                  arg_cache->type_info,
-                                                  iface_cache->g_type,
-                                                  iface_cache->py_type,
-                                                  arg_cache->transfer,
-                                                  arg_cache->is_caller_allocates,
-                                                  iface_cache->is_foreign);
+    return _pygi_marshal_from_py_interface_struct (py_arg,
+                                                   arg,
+                                                   arg_cache->arg_name,
+                                                   iface_cache->interface_info,
+                                                   arg_cache->type_info,
+                                                   iface_cache->g_type,
+                                                   iface_cache->py_type,
+                                                   arg_cache->transfer,
+                                                   TRUE, /*copy_reference*/
+                                                   iface_cache->is_foreign);
 }
 
 gboolean
@@ -1693,7 +1509,7 @@ _pygi_marshal_from_py_interface_object (PyGIInvokeState   *state,
         return FALSE;
     }
 
-    return pygi_marshal_from_py_gobject (py_arg, arg, arg_cache->transfer);
+    return _pygi_marshal_from_py_gobject (py_arg, arg, arg_cache->transfer);
 }
 
 gboolean
@@ -1785,14 +1601,14 @@ gboolean _pygi_marshal_from_py_interface_instance (PyGIInvokeState   *state,
    return TRUE;
 }
 
-/* pygi_marshal_from_py_gobject:
+/* _pygi_marshal_from_py_gobject:
  * py_arg: (in):
  * arg: (out):
  */
 gboolean
-pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
-                              GIArgument *arg,  /*out*/
-                              GITransfer transfer) {
+_pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
+                               GIArgument *arg,  /*out*/
+                               GITransfer transfer) {
     GObject *gobj;
 
     if (py_arg == Py_None) {
@@ -1810,12 +1626,38 @@ pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
 
     gobj = pygobject_get (py_arg);
     if (transfer == GI_TRANSFER_EVERYTHING) {
-        /* An easy case of adding a new ref that the caller will take ownership of.
-         * Pythons existing ref to the GObject will be managed normally with the wrapper.
+        /* For transfer everything, add a new ref that the callee will take ownership of.
+         * Pythons existing ref to the GObject will be managed with the PyGObject wrapper.
          */
         g_object_ref (gobj);
+    }
 
-    } else if (py_arg->ob_refcnt == 1 && gobj->ref_count == 1) {
+    arg->v_pointer = gobj;
+    return TRUE;
+}
+
+/* _pygi_marshal_from_py_gobject_out_arg:
+ * py_arg: (in):
+ * arg: (out):
+ *
+ * A specialization for marshaling Python GObjects used for out/return values
+ * from a Python implemented vfuncs, signals, or an assignment to a GObject property.
+ */
+gboolean
+_pygi_marshal_from_py_gobject_out_arg (PyObject *py_arg, /*in*/
+                                       GIArgument *arg,  /*out*/
+                                       GITransfer transfer) {
+    GObject *gobj;
+    if (!_pygi_marshal_from_py_gobject (py_arg, arg, transfer)) {
+        return FALSE;
+    }
+
+    /* HACK: At this point the basic marshaling of the GObject was successful
+     * but we add some special case hacks for vfunc returns due to buggy APIs:
+     * https://bugzilla.gnome.org/show_bug.cgi?id=693393
+     */
+    gobj = arg->v_pointer;
+    if (py_arg->ob_refcnt == 1 && gobj->ref_count == 1) {
         /* If both object ref counts are only 1 at this point (the reference held
          * in a return tuple), we assume the GObject will be free'd before reaching
          * its target and become invalid. So instead of getting invalid object errors
@@ -1824,12 +1666,10 @@ pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
         g_object_ref (gobj);
 
         if (((PyGObject *)py_arg)->private_flags.flags & PYGOBJECT_GOBJECT_WAS_FLOATING) {
-            /* HACK:
+            /*
              * We want to re-float instances that were floating and the Python
              * wrapper assumed ownership. With the additional caveat that there
              * are not any strong references beyond the return tuple.
-             * This should be removed once the following ticket is fixed:
-             * https://bugzilla.gnome.org/show_bug.cgi?id=693393
              */
             g_object_force_floating (gobj);
 
@@ -1848,21 +1688,21 @@ pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
         }
     }
 
-    arg->v_pointer = gobj;
     return TRUE;
 }
 
-/* pygi_marshal_from_py_gvalue:
+/* _pygi_marshal_from_py_gvalue:
  * py_arg: (in):
  * arg: (out):
  * transfer:
- * is_allocated: TRUE if arg->v_pointer is an already allocated GValue
+ * copy_reference: TRUE if arg should use the pointer reference held by py_arg
+ *                 when it is already holding a GValue vs. copying the value.
  */
 gboolean
-pygi_marshal_from_py_gvalue (PyObject *py_arg,
-                             GIArgument *arg,
-                             GITransfer transfer,
-                             gboolean is_allocated) {
+_pygi_marshal_from_py_gvalue (PyObject *py_arg,
+                              GIArgument *arg,
+                              GITransfer transfer,
+                              gboolean copy_reference) {
     GValue *value;
     GType object_type;
 
@@ -1872,24 +1712,21 @@ pygi_marshal_from_py_gvalue (PyObject *py_arg,
         return FALSE;
     }
 
-    if (is_allocated)
-        value = (GValue *)arg->v_pointer;
-    else
-        value = g_slice_new0 (GValue);
-
     /* if already a gvalue, use that, else marshal into gvalue */
     if (object_type == G_TYPE_VALUE) {
         GValue *source_value = pyg_boxed_get (py_arg, GValue);
-        if (G_VALUE_TYPE (value) == G_TYPE_INVALID)
+        if (copy_reference) {
+            value = source_value;
+        } else {
+            value = g_slice_new0 (GValue);
             g_value_init (value, G_VALUE_TYPE (source_value));
-        g_value_copy (source_value, value);
+            g_value_copy (source_value, value);
+        }
     } else {
-        if (G_VALUE_TYPE (value) == G_TYPE_INVALID)
-            g_value_init (value, object_type);
-
+        value = g_slice_new0 (GValue);
+        g_value_init (value, object_type);
         if (pyg_value_from_pyobject (value, py_arg) < 0) {
-            if (!is_allocated)
-                g_slice_free (GValue, value);
+            g_slice_free (GValue, value);
             PyErr_SetString (PyExc_RuntimeError, "PyObject conversion to GValue failed");
             return FALSE;
         }
@@ -1899,13 +1736,13 @@ pygi_marshal_from_py_gvalue (PyObject *py_arg,
     return TRUE;
 }
 
-/* pygi_marshal_from_py_gclosure:
+/* _pygi_marshal_from_py_gclosure:
  * py_arg: (in):
  * arg: (out):
  */
 gboolean
-pygi_marshal_from_py_gclosure(PyObject *py_arg,
-                              GIArgument *arg)
+_pygi_marshal_from_py_gclosure(PyObject *py_arg,
+                               GIArgument *arg)
 {
     GClosure *closure;
     GType object_gtype = pyg_type_from_object_strict (py_arg, FALSE);
@@ -1932,16 +1769,16 @@ pygi_marshal_from_py_gclosure(PyObject *py_arg,
 }
 
 gboolean
-pygi_marshal_from_py_interface_struct (PyObject *py_arg,
-                                       GIArgument *arg,
-                                       const gchar *arg_name,
-                                       GIBaseInfo *interface_info,
-                                       GITypeInfo *type_info,
-                                       GType g_type,
-                                       PyObject *py_type,
-                                       GITransfer transfer,
-                                       gboolean is_allocated,
-                                       gboolean is_foreign)
+_pygi_marshal_from_py_interface_struct (PyObject *py_arg,
+                                        GIArgument *arg,
+                                        const gchar *arg_name,
+                                        GIBaseInfo *interface_info,
+                                        GITypeInfo *type_info,
+                                        GType g_type,
+                                        PyObject *py_type,
+                                        GITransfer transfer,
+                                        gboolean copy_reference,
+                                        gboolean is_foreign)
 {
     if (py_arg == Py_None) {
         arg->v_pointer = NULL;
@@ -1953,12 +1790,12 @@ pygi_marshal_from_py_interface_struct (PyObject *py_arg,
      */
 
     if (g_type_is_a (g_type, G_TYPE_CLOSURE)) {
-        return pygi_marshal_from_py_gclosure (py_arg, arg);
+        return _pygi_marshal_from_py_gclosure (py_arg, arg);
     } else if (g_type_is_a (g_type, G_TYPE_VALUE)) {
-        return pygi_marshal_from_py_gvalue(py_arg,
-                                           arg,
-                                           transfer,
-                                           is_allocated);
+        return _pygi_marshal_from_py_gvalue(py_arg,
+                                            arg,
+                                            transfer,
+                                            copy_reference);
     } else if (is_foreign) {
         PyObject *success;
         success = pygi_struct_foreign_convert_to_g_argument (py_arg,
