@@ -33,6 +33,7 @@
 #include "pygi-cache.h"
 #include "pygi-marshal-cleanup.h"
 #include "pygi-marshal-to-py.h"
+#include "pygi-argument.h"
 
 static gboolean
 gi_argument_to_c_long (GIArgument *arg_in,
@@ -110,7 +111,6 @@ gi_argument_to_gsize (GIArgument *arg_in,
     }
 }
 
-
 PyObject *
 _pygi_marshal_to_py_void (PyGIInvokeState   *state,
                           PyGICallableCache *callable_cache,
@@ -118,139 +118,23 @@ _pygi_marshal_to_py_void (PyGIInvokeState   *state,
                           GIArgument        *arg)
 {
     PyObject *py_obj = NULL;
-    if (arg_cache->is_pointer)
+    if (arg_cache->is_pointer) {
+        /* NOTE: This will change to interpret pointers as integer values
+         * by using the following:
+         * py_obj = PyLong_FromVoidPtr (arg->v_pointer);
+         * See: https://bugzilla.gnome.org/show_bug.cgi?id=688081
+         */
         py_obj = arg->v_pointer;
-    else
+    } else {
         py_obj = Py_None;
+    }
 
     Py_XINCREF (py_obj);
     return py_obj;
 }
 
-PyObject *
-_pygi_marshal_to_py_boolean (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             GIArgument        *arg)
-{
-    PyObject *py_obj = PyBool_FromLong (arg->v_boolean);
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_int8 (PyGIInvokeState   *state,
-                          PyGICallableCache *callable_cache,
-                          PyGIArgCache      *arg_cache,
-                          GIArgument        *arg)
-{
-    PyObject *py_obj = PYGLIB_PyLong_FromLong (arg->v_int8);
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_uint8 (PyGIInvokeState   *state,
-                           PyGICallableCache *callable_cache,
-                           PyGIArgCache      *arg_cache,
-                           GIArgument        *arg)
-{
-    PyObject *py_obj =  PYGLIB_PyLong_FromLong (arg->v_uint8);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_int16 (PyGIInvokeState   *state,
-                           PyGICallableCache *callable_cache,
-                           PyGIArgCache      *arg_cache,
-                           GIArgument        *arg)
-{
-    PyObject *py_obj =  PYGLIB_PyLong_FromLong (arg->v_int16);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_uint16 (PyGIInvokeState   *state,
-                            PyGICallableCache *callable_cache,
-                            PyGIArgCache      *arg_cache,
-                            GIArgument        *arg)
-{
-    PyObject *py_obj =  PYGLIB_PyLong_FromLong (arg->v_uint16);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_int32 (PyGIInvokeState   *state,
-                           PyGICallableCache *callable_cache,
-                           PyGIArgCache      *arg_cache,
-                           GIArgument        *arg)
-{
-    PyObject *py_obj = PYGLIB_PyLong_FromLong (arg->v_int32);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_uint32 (PyGIInvokeState   *state,
-                            PyGICallableCache *callable_cache,
-                            PyGIArgCache      *arg_cache,
-                            GIArgument        *arg)
-{
-    PyObject *py_obj = PyLong_FromLongLong (arg->v_uint32);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_int64 (PyGIInvokeState   *state,
-                           PyGICallableCache *callable_cache,
-                           PyGIArgCache      *arg_cache,
-                           GIArgument        *arg)
-{
-    PyObject *py_obj = PyLong_FromLongLong (arg->v_int64);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_uint64 (PyGIInvokeState   *state,
-                            PyGICallableCache *callable_cache,
-                            PyGIArgCache      *arg_cache,
-                            GIArgument        *arg)
-{
-    PyObject *py_obj = PyLong_FromUnsignedLongLong (arg->v_uint64);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_float (PyGIInvokeState   *state,
-                           PyGICallableCache *callable_cache,
-                           PyGIArgCache      *arg_cache,
-                           GIArgument        *arg)
-{
-    PyObject *py_obj = PyFloat_FromDouble (arg->v_float);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_double (PyGIInvokeState   *state,
-                            PyGICallableCache *callable_cache,
-                            PyGIArgCache      *arg_cache,
-                            GIArgument        *arg)
-{
-    PyObject *py_obj = PyFloat_FromDouble (arg->v_double);
-
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_unichar (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             GIArgument        *arg)
+static PyObject *
+_pygi_marshal_to_py_unichar (GIArgument *arg)
 {
     PyObject *py_obj = NULL;
 
@@ -273,23 +157,8 @@ _pygi_marshal_to_py_unichar (PyGIInvokeState   *state,
     return py_obj;
 }
 
-PyObject *
-_pygi_marshal_to_py_gtype (PyGIInvokeState   *state,
-                           PyGICallableCache *callable_cache,
-                           PyGIArgCache      *arg_cache,
-                           GIArgument        *arg)
-{
-    PyObject *py_obj = NULL;
-
-    py_obj = pyg_type_wrapper_new ( (GType)arg->v_long);
-    return py_obj;
-}
-
-PyObject *
-_pygi_marshal_to_py_utf8 (PyGIInvokeState   *state,
-                          PyGICallableCache *callable_cache,
-                          PyGIArgCache      *arg_cache,
-                          GIArgument        *arg)
+static PyObject *
+_pygi_marshal_to_py_utf8 (GIArgument *arg)
 {
     PyObject *py_obj = NULL;
     if (arg->v_string == NULL) {
@@ -300,11 +169,8 @@ _pygi_marshal_to_py_utf8 (PyGIInvokeState   *state,
     return py_obj;
 }
 
-PyObject *
-_pygi_marshal_to_py_filename (PyGIInvokeState   *state,
-                              PyGICallableCache *callable_cache,
-                              PyGIArgCache      *arg_cache,
-                              GIArgument        *arg)
+static PyObject *
+_pygi_marshal_to_py_filename (GIArgument *arg)
 {
     gchar *string = NULL;
     PyObject *py_obj = NULL;
@@ -327,6 +193,89 @@ _pygi_marshal_to_py_filename (PyGIInvokeState   *state,
     return py_obj;
 }
 
+
+/**
+ * _pygi_marshal_to_py_basic_type:
+ * @arg: The argument to convert to an object.
+ * @type_tag: Type tag for @arg
+ * @transfer: Transfer annotation
+ *
+ * Convert the given argument to a Python object. This function
+ * is restricted to simple types that only require the GITypeTag
+ * and GITransfer. For a more complete conversion routine, use:
+ * _pygi_argument_to_object.
+ *
+ * Returns: A PyObject representing @arg or NULL if it cannot convert
+ *          the argument.
+ */
+PyObject *
+_pygi_marshal_to_py_basic_type (GIArgument  *arg,
+                                 GITypeTag type_tag,
+                                 GITransfer transfer)
+{
+    switch (type_tag) {
+        case GI_TYPE_TAG_BOOLEAN:
+            return PyBool_FromLong (arg->v_boolean);
+
+        case GI_TYPE_TAG_INT8:
+            return PYGLIB_PyLong_FromLong (arg->v_int8);
+
+        case GI_TYPE_TAG_UINT8:
+            return PYGLIB_PyLong_FromLong (arg->v_uint8);
+
+        case GI_TYPE_TAG_INT16:
+            return PYGLIB_PyLong_FromLong (arg->v_int16);
+
+        case GI_TYPE_TAG_UINT16:
+            return PYGLIB_PyLong_FromLong (arg->v_uint16);
+
+        case GI_TYPE_TAG_INT32:
+            return PYGLIB_PyLong_FromLong (arg->v_int32);
+
+        case GI_TYPE_TAG_UINT32:
+            return PyLong_FromLongLong (arg->v_uint32);
+
+        case GI_TYPE_TAG_INT64:
+            return PyLong_FromLongLong (arg->v_int64);
+
+        case GI_TYPE_TAG_UINT64:
+            return PyLong_FromUnsignedLongLong (arg->v_uint64);
+
+        case GI_TYPE_TAG_FLOAT:
+            return PyFloat_FromDouble (arg->v_float);
+
+        case GI_TYPE_TAG_DOUBLE:
+            return PyFloat_FromDouble (arg->v_double);
+
+        case GI_TYPE_TAG_GTYPE:
+            return pyg_type_wrapper_new ( (GType) arg->v_long);
+
+        case GI_TYPE_TAG_UNICHAR:
+            return _pygi_marshal_to_py_unichar (arg);
+
+        case GI_TYPE_TAG_UTF8:
+            return _pygi_marshal_to_py_utf8 (arg);
+
+        case GI_TYPE_TAG_FILENAME:
+            return _pygi_marshal_to_py_filename (arg);
+
+        default:
+            return NULL;
+    }
+    return NULL;
+}
+
+PyObject *
+_pygi_marshal_to_py_basic_type_cache_adapter (PyGIInvokeState   *state,
+                                              PyGICallableCache *callable_cache,
+                                              PyGIArgCache      *arg_cache,
+                                              GIArgument        *arg)
+{
+    return _pygi_marshal_to_py_basic_type (arg,
+                                            arg_cache->type_tag,
+                                            arg_cache->transfer);
+}
+
 PyObject *
 _pygi_marshal_to_py_array (PyGIInvokeState   *state,
                            PyGICallableCache *callable_cache,
@@ -740,6 +689,7 @@ _pygi_marshal_to_py_interface_enum (PyGIInvokeState   *state,
     } else {
         py_obj = pyg_enum_from_gtype (iface_cache->g_type, c_long);
     }
+    g_base_info_unref (interface);
     return py_obj;
 }
 
@@ -759,9 +709,11 @@ _pygi_marshal_to_py_interface_flags (PyGIInvokeState   *state,
 
     if (!gi_argument_to_c_long(arg, &c_long,
                                g_enum_info_get_storage_type ((GIEnumInfo *)interface))) {
+        g_base_info_unref (interface);
         return NULL;
     }
 
+    g_base_info_unref (interface);
     if (iface_cache->g_type == G_TYPE_NONE) {
         /* An enum with a GType of None is an enum without GType */
 
@@ -790,20 +742,20 @@ _pygi_marshal_to_py_interface_flags (PyGIInvokeState   *state,
 }
 
 PyObject *
-_pygi_marshal_to_py_interface_struct (PyGIInvokeState   *state,
-                                      PyGICallableCache *callable_cache,
-                                      PyGIArgCache      *arg_cache,
-                                      GIArgument        *arg)
+_pygi_marshal_to_py_interface_struct_cache_adapter (PyGIInvokeState   *state,
+                                                    PyGICallableCache *callable_cache,
+                                                    PyGIArgCache      *arg_cache,
+                                                    GIArgument        *arg)
 {
     PyGIInterfaceCache *iface_cache = (PyGIInterfaceCache *)arg_cache;
 
-    return pygi_marshal_to_py_interface_struct (arg,
-                                                iface_cache->interface_info,
-                                                iface_cache->g_type,
-                                                iface_cache->py_type,
-                                                arg_cache->transfer,
-                                                arg_cache->is_caller_allocates,
-                                                iface_cache->is_foreign);
+    return _pygi_marshal_to_py_interface_struct (arg,
+                                                 iface_cache->interface_info,
+                                                 iface_cache->g_type,
+                                                 iface_cache->py_type,
+                                                 arg_cache->transfer,
+                                                 arg_cache->is_caller_allocates,
+                                                 iface_cache->is_foreign);
 }
 
 PyObject *
@@ -833,12 +785,12 @@ _pygi_marshal_to_py_interface_boxed (PyGIInvokeState   *state,
 }
 
 PyObject *
-_pygi_marshal_to_py_interface_object (PyGIInvokeState   *state,
-                                      PyGICallableCache *callable_cache,
-                                      PyGIArgCache      *arg_cache,
-                                      GIArgument        *arg)
+_pygi_marshal_to_py_interface_object_cache_adapter (PyGIInvokeState   *state,
+                                                    PyGICallableCache *callable_cache,
+                                                    PyGIArgCache      *arg_cache,
+                                                    GIArgument        *arg)
 {
-    return pygi_marshal_to_py_object(arg, arg_cache->transfer);
+    return _pygi_marshal_to_py_object(arg, arg_cache->transfer);
 }
 
 PyObject *
@@ -855,7 +807,7 @@ _pygi_marshal_to_py_interface_union  (PyGIInvokeState   *state,
 }
 
 PyObject *
-pygi_marshal_to_py_object (GIArgument *arg, GITransfer transfer) {
+_pygi_marshal_to_py_object (GIArgument *arg, GITransfer transfer) {
     PyObject *pyobj;
 
     if (arg->v_pointer == NULL) {
@@ -877,13 +829,13 @@ pygi_marshal_to_py_object (GIArgument *arg, GITransfer transfer) {
 }
 
 PyObject *
-pygi_marshal_to_py_interface_struct (GIArgument *arg,
-                                     GIInterfaceInfo *interface_info,
-                                     GType g_type,
-                                     PyObject *py_type,
-                                     GITransfer transfer,
-                                     gboolean is_allocated,
-                                     gboolean is_foreign)
+_pygi_marshal_to_py_interface_struct (GIArgument *arg,
+                                      GIInterfaceInfo *interface_info,
+                                      GType g_type,
+                                      PyObject *py_type,
+                                      GITransfer transfer,
+                                      gboolean is_allocated,
+                                      gboolean is_foreign)
 {
     PyObject *py_obj = NULL;
 
